# spark_quality_rules_tools


[![Github License](https://img.shields.io/badge/License-Apache%202.0-blue.svg)](https://opensource.org/licenses/Apache-2.0)
[![Updates](https://pyup.io/repos/github/woctezuma/google-colab-transfer/shield.svg)](pyup)
[![Python 3](https://pyup.io/repos/github/woctezuma/google-colab-transfer/python-3-shield.svg)](pyup)
[![Code coverage](https://codecov.io/gh/woctezuma/google-colab-transfer/branch/master/graph/badge.svg)](codecov)




spark_quality_rules_tools is a Python library that implements quality rules in sandbox
## Installation

The code is packaged for PyPI, so that the installation consists in running:
```sh
pip install spark-quality-rules-tools 
```


## Usage

wrapper run hammurabies

```sh
from spark_quality_rules_tools import show_spark_df
from spark_quality_rules_tools import show_pd_df
from spark_quality_rules_tools import get_path_workspace
from spark_quality_rules_tools import quality_rules_download_jar
from spark_quality_rules_tools import quality_rules_extract_parameters
from spark_quality_rules_tools import quality_rules_run_execution
from spark_quality_rules_tools import quality_rules_hammurabi
from spark_quality_rules_tools import BASE_DIR 
from pyspark.sql import SparkSession
import os
```

```sh
username = "jonap"
os.environ['UUAA_CODE'] = f"finan/data/projects/CDD/{username}"
os.environ['JPY_USER'] = 'P0XXXXX'
```

```sh
spark = SparkSession.builder \
        .master("local[*]") \
        .appName("JONAP") \
        .getOrCreate()
sc = spark.sparkContext
sc._conf.setExecutorEnv("UUAA_CODE", os.getenv("UUAA_CODE"))
sc._conf.setExecutorEnv("JPY_USER", os.getenv("JPY_USER"))
```

##QR Download Jar
```sh
# sandbox
quality_rules_download_jar(user_sandbox="P030772",
                           jar_url="https://file.io/HTUDZqqrGdW2")
#local
quality_rules_download_jar(user_sandbox="P030772",
                           jar_url="https://file.io/Lkix8Kz03ttO",
                           type_sandbox="local")
```

##QR Extract Parameters
```sh
conf_list = [
"https://gist.githubusercontent.com/jonaqp/ee73ac37d7d34d07ec17b1f8972512fc/raw/a456bfc9ee27a360754114fd8204997752e2a127/dq_ksan_eomcontract_master_eomcontract-01.conf",
"https://gist.githubusercontent.com/jonaqp/8e93897c6b2c3c91dd1ae19684f5b2cb/raw/9fcc35e45cd0a59b94fe2b633eec5ebea39d060d/dq_psan_eomassetsliabilities_master_eomassetsliabilities-01.conf",
"https://gist.githubusercontent.com/jonaqp/93b8acf8750cb4c5c5a87f0d63be7504/raw/089e14b0275dadbfa6cefcb5842a6f256d12c1d3/dq_ksan_eominstlmntplan_master_eominstlmntplan-01.conf"    
]

#sandbox
quality_rules_extract_parameters(conf_lists=conf_list,
                                 user_sandbox=os.getenv("JPY_USER"))

#local
quality_rules_extract_parameters(conf_lists=conf_list,
                                 user_sandbox=os.getenv("JPY_USER"),
                                 type_sandbox="local")  
                                                                                                                             
```

##QR Run Execution
```sh
conf_lists = [
"dq_ksan_eomcontract_master_eomcontract-01.conf",
"dq_ksan_eominstlmntplan_master_eominstlmntplan-01.conf"    
]

#sandbox
quality_rules_extract_parameters(conf_lists=conf_list,
                                 user_sandbox=os.getenv("JPY_USER"))
                                 
 #local                                
quality_rules_run_execution(spark=spark,
                            conf_lists=conf_lists,
                            user_sandbox=os.getenv("JPY_USER"),
                            uuaa_name="ksan",
                            uuaa_code=os.getenv("UUAA_CODE"),
                            type_sandbox="local")
```

## License

[Apache License 2.0](https://www.dropbox.com/s/8t6xtgk06o3ij61/LICENSE?dl=0).


## New features v1.0

 
## BugFix
- choco install visualcpp-build-tools



## Reference

 - Jonathan Quiza [github](https://github.com/jonaqp).
 - Jonathan Quiza [RumiMLSpark](http://rumi-ml.herokuapp.com/).
 - Jonathan Quiza [linkedin](https://www.linkedin.com/in/jonaqp/).
