import warnings
warnings.filterwarnings('always')
warnings.filterwarnings('ignore')

def get_path_workspace(type_sandbox=None, user_sandbox=None):
    if str(type_sandbox).lower() == "local":
        path_workspace = ""
    else:
        path_workspace = f"/var/sds/homes/{user_sandbox}/workspace/"
    return path_workspace


def quality_rules_download_jar(user_sandbox=None,
                               jar_url=None,
                               type_sandbox=None):
    import requests
    import sys
    import os

    if not os.getenv('JPY_USER'):
        if not user_sandbox:
            raise Exception(f'required variable user_sandbox')
    else:
        user_sandbox = os.getenv("JPY_USER")

    if jar_url is None:
        jar_url = "http://artifactory-flegetonte.live.mx.ether.igrupobbva/artifactory/list/" \
                  "gl-analytics-artifacts-data-generic/jars/hammurabi-sandbox-4.6.1-jar-with-dependencies.jar",

    path_workspace = get_path_workspace(type_sandbox=type_sandbox, user_sandbox=user_sandbox)

    is_windows = sys.platform.startswith('win')
    jar_name = "hammurabi-sandbox-spark3.jar"
    directory_output_python = os.path.join(path_workspace, "artifacts", "python")

    if is_windows:
        directory_output_python = os.path.join(directory_output_python).replace("\\", "/")
        directory_output_python_jar_file = os.path.join(directory_output_python, jar_name).replace("\\", "/")
    else:
        directory_output_python = directory_output_python
        directory_output_python_jar_file = os.path.join(directory_output_python, jar_name)

    if not os.path.exists(f'{directory_output_python}'):
        os.makedirs(f'{directory_output_python}')

    if os.path.isfile(directory_output_python_jar_file):
        os.remove(directory_output_python_jar_file)

    with requests.get(jar_url, stream=True, verify=True) as r:
        r.raise_for_status()
        os.makedirs(os.path.dirname(directory_output_python), exist_ok=True)
        with open(directory_output_python_jar_file, 'wb+') as f:
            for chunk in r.iter_content(chunk_size=8192):
                f.write(chunk)
    print(f"Download finished on {directory_output_python_jar_file}.")


def quality_rules_extract_parameters(conf_lists=None,
                                     user_sandbox=None,
                                     type_sandbox=None):
    import requests
    import os
    import sys
    import json
    import re

    if not conf_lists:
        raise Exception(f'required variable urls hammurabi conf')

    if not os.getenv('JPY_USER'):
        if not user_sandbox:
            raise Exception(f'required variable user_sandbox')
        else:
            user_sandbox = user_sandbox
    else:
        user_sandbox = os.getenv("JPY_USER")

    is_windows = sys.platform.startswith('win')
    path_workspace = get_path_workspace(type_sandbox=type_sandbox, user_sandbox=user_sandbox)

    # generated conf
    rs_dict = dict()
    for conf_url in conf_lists:
        conf_name = str(conf_url).split("/")[-1]
        uuaa_name = str(conf_url).split("_")[-4]
        table_name = str(conf_name.split(".")[0])
        directory_output_confs = os.path.join(path_workspace, "dq_conf", f"{uuaa_name}", "conf")

        if is_windows:
            directory_output_confs = directory_output_confs.replace("\\", "/")
            directory_output_confs_file = os.path.join(directory_output_confs, conf_name).replace("\\", "/")
        else:
            directory_output_confs = directory_output_confs
            directory_output_confs_file = os.path.join(directory_output_confs, conf_name)

        if not os.path.exists(f'{directory_output_confs}'):
            os.makedirs(f'{directory_output_confs}')

        read_file = requests.get(conf_url, allow_redirects=True)
        with open(f'{directory_output_confs_file}', 'wb') as f:
            f.write(read_file.content)
        with open(f'{directory_output_confs_file}') as f:
            hammurabi_conf = f.read()

        variables_list = sorted(list(set(re.findall(r'{(\w+)}', hammurabi_conf))))
        variables_dict = {variables: "" for variables in variables_list}

        if uuaa_name not in rs_dict.keys():
            rs_dict[uuaa_name] = list()
        rs_dict[uuaa_name].append({f"{table_name}": variables_dict})

    # generated parameter
    for uuaa_name, values in rs_dict.items():
        rs_dict2 = dict()
        rs_dict2[uuaa_name] = values
        directory_output_parameters = os.path.join(path_workspace, "dq_conf", f"{uuaa_name}", "parameter")

        if is_windows:
            directory_output_parameters = directory_output_parameters.replace("\\", "/")
            directory_output_parameters_file = os.path.join(directory_output_parameters, "parameters.json").replace("\\", "/")
        else:
            directory_output_parameters = directory_output_parameters
            directory_output_parameters_file = os.path.join(directory_output_parameters, "parameters.json")

        if not os.path.exists(f'{directory_output_parameters}'):
            os.makedirs(f'{directory_output_parameters}')

        with open(f"{directory_output_parameters_file}", "w") as f:
            json.dump(rs_dict2, f, indent=4)

        print(f"Generating a configuration file: {directory_output_parameters_file}")


def quality_rules_hammurabi(spark=None,
                            name_conf=None,
                            conf_filename=None,
                            uuaa_code=None,
                            user_sandbox=None,
                            now=None):
    from spark_quality_rules_tools.utils.dataframe import show_pd_df
    from spark_quality_rules_tools.utils.dataframe import show_spark_df
    from pyspark.sql import functions as func
    import pyspark
    import pandas as pd
    import sys
    import os

    pyspark.sql.dataframe.DataFrame.show2 = show_spark_df
    pd.DataFrame.show2 = show_pd_df

    sc = spark.sparkContext
    conf = sc._jvm.java.io.File(conf_filename)
    ConfigFactory = sc._jvm.com.typesafe.config.ConfigFactory
    parsed_conf = ConfigFactory.parseFile(conf)
    Hammurabi = sc._jvm.com.datio.hammurabi.sandbox.Hammurabi
    result = Hammurabi.run(spark._jsparkSession, parsed_conf)

    is_windows = sys.platform.startswith('win')
    current_datetime = now.strftime("%Y-%m-%d %H:%M:%S")
    current_datetime_str = now.strftime("%Y%m%d%H%M")

    if is_windows:
        path_folder_dq = os.path.join(f"/data/sandboxes/{uuaa_code}/data/users/{user_sandbox}/dq/").replace("\\", "/")
        path_folder_metrics = os.path.join(path_folder_dq, "metrics").replace("\\", "/")
        path_folder_reports = os.path.join(path_folder_dq, "reports").replace("\\", "/")
        path_folder_reports_filename = os.path.join(path_folder_reports, f"{name_conf}_{current_datetime_str}.csv").replace("\\", "/")
    else:
        path_folder_dq = os.path.join(f"/data/sandboxes/{uuaa_code}/data/users/{user_sandbox}/dq/")
        path_folder_metrics = os.path.join(path_folder_dq, "metrics")
        path_folder_reports = os.path.join(path_folder_dq, "reports")
        path_folder_reports_filename = os.path.join(path_folder_reports, f"{name_conf}_{current_datetime_str}.csv")

    if not os.path.exists(f'{path_folder_metrics}'):
        os.makedirs(f'{path_folder_metrics}')

    if not os.path.exists(f'{path_folder_reports}'):
        os.makedirs(f'{path_folder_reports}')

    metrics_df = spark.read.parquet(f'{path_folder_metrics}/', inferschema=True)
    metrics_filter = metrics_df.filter(func.col("gf_quality_rule_execution_date") > func.unix_timestamp(func.lit(current_datetime)).cast('timestamp'))
    metrics_filter_pandas = metrics_filter.toPandas()
    metrics_filter_pandas.to_csv(path_folder_reports_filename, index=False)
    print(f"Generating a file csv : {path_folder_reports_filename}")


def quality_rules_run_execution(spark=None,
                                conf_lists=None,
                                user_sandbox=None,
                                uuaa_name=None,
                                uuaa_code=None,
                                type_sandbox=None):
    import os
    import sys
    import json
    from pyhocon import ConfigFactory
    from pyhocon.converter import HOCONConverter
    from datetime import datetime

    if not os.getenv('JPY_USER'):
        if not user_sandbox:
            raise Exception(f'required variable user_sandbox')
    else:
        user_sandbox = os.getenv("JPY_USER")

    is_windows = sys.platform.startswith('win')
    path_workspace = get_path_workspace(type_sandbox=type_sandbox, user_sandbox=user_sandbox)

    directory_output_parameters = os.path.join(path_workspace, "dq_conf", f"{uuaa_name}", "parameter")
    directory_output_confs = os.path.join(path_workspace, "dq_conf", f"{uuaa_name}", "conf")
    directory_output_hocons = os.path.join(path_workspace, "dq_conf", f"{uuaa_name}", "hocon")

    if is_windows:
        directory_output_parameters_file = os.path.join(directory_output_parameters, "parameters.json").replace("\\", "/")
        directory_output_confs = directory_output_confs.replace("\\", "/")
        directory_output_hocons = directory_output_hocons.replace("\\", "/")
    else:
        directory_output_parameters_file = os.path.join(directory_output_parameters, "parameters.json")
        directory_output_confs = directory_output_confs
        directory_output_hocons = directory_output_hocons

    if not os.path.exists(f'{directory_output_confs}'):
        os.makedirs(f'{directory_output_confs}')
    if not os.path.exists(f'{directory_output_hocons}'):
        os.makedirs(f'{directory_output_hocons}')

    with open(f'{directory_output_parameters_file}') as f:
        parameter_conf = f.read()
    json_string = json.loads(parameter_conf)

    list_conf_tables = [_ for k in json_string[uuaa_name] for _ in k if f"{_}.conf" in conf_lists]
    list_json_tables_parameter = {k1: v1 for k in json_string[uuaa_name] for k1, v1 in k.items()}

    rs_directory_list = list()
    for name_conf in list_conf_tables:
        if is_windows:
            directory_output_confs_file = os.path.join(directory_output_confs, f"{name_conf}.conf").replace("\\", "/")
            directory_output_hocons_file = os.path.join(directory_output_hocons, f"{name_conf}.conf").replace("\\", "/")
        else:
            directory_output_confs_file = os.path.join(directory_output_confs, f"{name_conf}.conf")
            directory_output_hocons_file = os.path.join(directory_output_hocons, f"{name_conf}.conf")

        with open(f'{directory_output_confs_file}') as f:
            current_conf = f.read()

        for k, v in list_json_tables_parameter[name_conf].items():
            current_conf = current_conf.replace(f'${{{k}}}', v)
        current_conf = current_conf.replace('/"/', '/')
        current_conf = current_conf.replace('/"', '/')

        with open(f"{directory_output_hocons_file}", "w") as f:
            f.write(current_conf)

        conf_file = ConfigFactory.parse_string(current_conf)
        hocons_file = HOCONConverter.to_hocon(conf_file)
        with open(directory_output_hocons_file, "w") as g:
            g.write(hocons_file)
        rs_directory_list.append(directory_output_hocons_file)
        print(f"Generating a configuration file hocon : {directory_output_hocons_file}")

        now = datetime.now()
        quality_rules_hammurabi(spark=spark,
                                name_conf=name_conf,
                                conf_filename=directory_output_hocons_file,
                                uuaa_code=uuaa_code,
                                user_sandbox=user_sandbox,
                                now=now)
